% based on markov model of Pan's Na m channel
% work in SECONDS not ms.

clear;
%clc;
% close all;

%% Options
run_optimisation = false;

%% Set directories
current_dir = pwd;
main_dir = current_dir; %
data_dir = [main_dir '\data' filesep];
output_dir = [main_dir '\output' filesep];
storage_dir = [main_dir '\storage' filesep];

%% Steady-state gating parameters and time constants

V = transpose(-120:1:60); % unit mV
V = transpose(-110:1:40); % unit mV

alpha_s = 0.0003442*exp(V/-17.6345);
beta_s = 0.064289*exp(V/15.2605);

tau_s = 1./(alpha_s + beta_s);
g_ss_s = alpha_s./(alpha_s + beta_s);


%% Fit bond graph parameters to model
% params: [kf, zf, kr, zr];
% Kf corresponds to k_s20: 1*alpha_s
error_func_alpha = @(params) square_error(alpha_s - calc_alpha(params,V/1000)); % fit to alpha - k.exp(zFV/RT) : do lsqminerror.
error_func_beta = @(params) square_error(beta_s - calc_beta(params,V/1000));
error_func_gss = @(params) square_error(g_ss_s - p2gss(params,V));
error_func_tau = @(params) square_error(tau_s- p2tau(params,V));

error_func = @(params) error_func_alpha(params) + error_func_beta(params) + error_func_gss(params) + error_func_tau(params);

A = [];
b = [];
Aeq = [];
beq = [];
lb = [0; -10; 0; -10];
ub = [Inf; 10; Inf; 10];

options_unc = optimoptions('fminunc','MaxFunEvals',10000);
options_ps = optimoptions('particleswarm','UseParallel',false,'HybridFcn',@fmincon,...
    'SwarmSize',1500,'FunctionTolerance',1e-14);

if run_optimisation
    [params_vec,fval,exitflag,output] = particleswarm(error_func,4,lb,ub,options_ps);
    save([storage_dir 'to_s_parameters.mat'],'params_vec');
else
    load([storage_dir 'to_s_parameters.mat']);
end
% [params_vec,fval,exitflag,output,grad,hessian] = fminunc(error_func,params_vec,options_unc);


g_ss_fit = p2gss(params_vec,V);
h1 = figure;
plot(V,g_ss_s,'kx');
hold on;
plot(V,g_ss_fit,'k');
xlabel('Voltage (mV)');
ylabel('m_{ss}');
legend('original','Fitted');
set(gca,'FontSize',28);
xlim([-120 60]);
set(gca,'XTick',-120:30:60);
set(gca,'YTick',0:0.2:1);
xticklabels({-120,'',-60,'',0,'',60});
yticklabels({0,'','','','',1});
set(gca,'LineWidth',3);
grid on;

tau_fit = p2tau(params_vec,V);
h2 = figure;
plot(V,tau_s,'kx');
hold on;
plot(V,tau_fit,'k');
legend('original','Fitted');
xlabel('Voltage (mV)');
ylabel('\tau_s (ms)');
set(gca,'FontSize',28);
xlim([-120 60]);
set(gca,'XTick',-120:30:60);
% set(gca,'YTick',0:0.2:1);
xticklabels({-120,'',-60,'',0,'',60});
% yticklabels({0,'','','','',1});
set(gca,'LineWidth',3);
set(gca,'xgrid','on');

alpha_fit = calc_alpha(params_vec,V/1000);
beta_fit = calc_beta(params_vec,V/1000);
figure,
subplot(1,2,1)
plot(V,alpha_s,'x',V,alpha_fit);
legend('original','fit');
subplot(1,2,2)
plot(V,beta_s,'x',V,beta_fit);
legend('original','fit');

print_figure(h1,output_dir,'g_ss_s');
print_figure(h2,output_dir,'tau_s');


